/*********************************************************************
 * File:   filter.c  (part of TTYTALK)
 * Author: Victor Volkman
 * Date:   June, 1988
 ********************************************************************/
#include "tty.h"

extern int c_row, c_col;
extern enum cmdtype mode;
extern FILE *capture_file;
extern FILE *send_file;
extern int term_vals[];
extern char *term_strs[];
extern int capture_mode;

char *crlf_line =    "\015\012";    /* CR and LF          */
char *blankex_line = " \015\012";   /* ...w/leading blank */


void xlat_char_in(c)    /* Apply input filters and display character */
int c;
{
    if (term_vals[CMD_INFILTER]) {   /* apply filter */
        c &= 0x7F;        /* strip hi bit */
        if (((c < CHAR_BLANK) && (term_strs[(int)CMD_FILTER][c] == '+')) ||
            (c >= CHAR_BLANK))
            disp_char(c);
    }
    else
        disp_char(c);

}


int xlat_char_out()   /* Filter an output line/char */
{
    static char inp_line[80];
    static char outp_line[256];
    static char *cur_char=NULL;
    char *out_char;
    char *trans_char;
    int line_pos;
    int c;
    int i;

    short len;

    if (*cur_char!='\0' && cur_char!=NULL)
        c = *cur_char++;

    else if ((cur_char=fgets(inp_line, sizeof(inp_line)-1, send_file)) != NULL) {
        len = strlen(inp_line);
        if (term_vals[CMD_OUTFILTR])         /* Use only CR */
            inp_line[len-1] = CHAR_CR;
        else
            strcpy(&inp_line[len-1],crlf_line);   /* Place CR and LF marks */

        /*  for (trans_char=inp_line; *trans_char!='\0'; trans_char++)
        printf("%2X ",*trans_char); */

        out_char = outp_line;
        if (term_vals[CMD_UCONLY])       /* Transform into upper case */
            for (trans_char=inp_line; *trans_char!='\0'; trans_char++) {
                if (*trans_char>='a' && *trans_char<='z')
                    *trans_char -= 'a' - 'A';
            }

        if (term_vals[CMD_BLANKEX])      /* Convert blank lines to have 1 blank */
            if (*inp_line==CHAR_CR)
                strcpy(inp_line,blankex_line);

        if (term_vals[CMD_TABEX]) {      /* Convert tabs to blanks */
            puts("doing tabex");
            out_char = outp_line;
            line_pos = 0;
            for (trans_char=inp_line; *trans_char!='\0'; trans_char++)
                if (*trans_char==CHAR_TAB) {
                    for (i=0; i < (line_pos % 8); i++)
                        *out_char++ = CHAR_BLANK;
                    line_pos += i-1;
                }
                else {
                    *out_char++ = *trans_char;
                    line_pos++;
                }
        }
        else
            strcpy(outp_line,inp_line);
        cur_char = outp_line;
        c = *cur_char++;
    }

    else
        c = EOF;
    return c;
}


void send_file_char()     /* Apply output filter and transmit */
{
    int c;

    if ( (c=xlat_char_out() )  != EOF)
        xmt_char(c);
    else {
        fclose(send_file);
        mode = CMD_NONE;
    }
}


void disp_char(c)     /* display actual character and handle screen */
int c;
{
    int scroll=FALSE;

    if (capture_mode==TRUE) fputc(c,capture_file);
#if defined MSC
#define stdprt stdprn
#endif
    if (term_vals[CMD_PRINTER]) fputc(c,stdprt);

    if (c==CHAR_LF)
        scroll = TRUE;
    else if (c==CHAR_CR)
        scroll = term_vals[CMD_LFAUTO];
    else if (c==CHAR_BS) {
        if (c_col)
            vid_write(CHAR_BLANK,c_row,--c_col,ATTR_TERM);
    }
    else
        vid_write(c,c_row,c_col++,ATTR_TERM);

    if ((c_col>WINDOW_COLS) || scroll) {
        if (c_row+1 >= WINDOW_ROWS)     /* scrolling reqd */
            vid_scroll(0,0,WINDOW_ROWS-1,WINDOW_COLS-1,1);
        else
            c_row++;
        c_col = 0;
        if (capture_mode==TRUE && term_vals[CMD_LFAUTO])
            fputc(CHAR_LF,capture_file);
    }
    vid_setrc(c_row,c_col);
}

